/* Copyright 2009 IHE International (http://www.ihe.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.ihe.gazelle.proxy.dicom;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.UnsupportedEncodingException;

import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.w3c.dom.Document;

import com.pixelmed.dicom.AttributeList;
import com.pixelmed.dicom.DicomException;
import com.pixelmed.dicom.DicomInputStream;
import com.pixelmed.dicom.XMLRepresentationOfDicomObjectFactory;
import com.pixelmed.validate.DicomSRValidator;

/**
 * 
 * @author abderrazek boufahja
 * 
 */
public class Pixelmed {

	public Pixelmed() {
	}

	public String getDumpResult(byte[] dataSet) throws IOException, DicomException {
		AttributeList list = new AttributeList();
		ByteArrayInputStream bis = new ByteArrayInputStream(dataSet);
		DicomInputStream dis = new DicomInputStream(bis);
		list.read(dis);
		return getDumpResultFromList(list);
	}

	public String getDumpResult(String dicompath) throws IOException, DicomException {
		AttributeList list = new AttributeList();
		list.read(dicompath);
		return getDumpResultFromList(list);
	}

	private String getDumpResultFromList(AttributeList list) {
		String a = list.toString();
		String[] listLign = a.split("\n");
		StringBuffer toView = new StringBuffer();
		String tab = "";
		for (int i = 0; i < listLign.length; i++) {
			if ((listLign[i].startsWith("%seq")) || (listLign[i].startsWith("%item"))) {
				tab = tab + "----";
			} else if (listLign[i].startsWith("%end")) {
				if (tab.length() > 3) {
					tab = tab.substring(0, tab.length() - 4);
				}
			} else {
				toView = toView.append(tab + listLign[i] + "\n");
			}
		}
		return toView.toString();
	}

	public String dicom2xml(String dicompath) throws IOException, DicomException, TransformerConfigurationException,
			TransformerException, ParserConfigurationException {
		AttributeList list = new AttributeList();
		list.read(dicompath);
		Document document = new XMLRepresentationOfDicomObjectFactory().getDocument(list);
		ByteArrayOutputStream outStream = new ByteArrayOutputStream();
		XMLRepresentationOfDicomObjectFactory.write(outStream, document);
		String maChaine = outStream.toString();
		// maChaine = XmlFormatter.prettyFormat(maChaine, 4);
		return maChaine;
	}

	public String validateDicomSRDose(String dicompath) throws UnsupportedEncodingException,
			ParserConfigurationException, TransformerException, IOException, DicomException {
		DicomSRValidator dval = new DicomSRValidator();
		AttributeList list = new AttributeList();
		list.read(dicompath, null, true, true);
		return dval.validate(list);
	}

}
