package net.ihe.gazelle.proxy.dicom;

/*
 * Copyright 2009 IHE International (http://www.ihe.net)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import java.awt.Container;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.RenderingHints;
import java.awt.Toolkit;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Iterator;

import javax.imageio.IIOImage;
import javax.imageio.ImageIO;
import javax.imageio.ImageWriteParam;
import javax.imageio.ImageWriter;
import javax.imageio.plugins.jpeg.JPEGImageWriteParam;
import javax.imageio.stream.ImageOutputStream;

/**
 * @author Abderrazek Boufahja > INRIA Rennes IHE development Project
 * 
 */
public class ThumbnailCreator {
	public static void createThumbnail(String filename, int thumbWidth, int thumbHeight, int quality, String outFilename)
			throws InterruptedException, FileNotFoundException, IOException {

		Image image = Toolkit.getDefaultToolkit().getImage(filename);
		MediaTracker mediaTracker = new MediaTracker(new Container());
		mediaTracker.addImage(image, 0);
		mediaTracker.waitForID(0);

		double thumbRatio = (double) thumbWidth / (double) thumbHeight;
		int imageWidth = image.getWidth(null);
		int imageHeight = image.getHeight(null);
		double imageRatio = (double) imageWidth / (double) imageHeight;
		if (thumbRatio < imageRatio) {
			thumbHeight = (int) (thumbWidth / imageRatio);
		} else {
			thumbWidth = (int) (thumbHeight * imageRatio);
		}

		BufferedImage thumbImage = new BufferedImage(thumbWidth, thumbHeight, BufferedImage.TYPE_INT_RGB);
		Graphics2D graphics2D = thumbImage.createGraphics();
		graphics2D.setRenderingHint(RenderingHints.KEY_INTERPOLATION, RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		graphics2D.drawImage(image, 0, 0, thumbWidth, thumbHeight, null);

		writeImage(thumbImage, new File(outFilename));
	}


	private static void writeImage(BufferedImage image, File imageFile) throws IOException {
		FileOutputStream fos = new FileOutputStream(imageFile);

		ImageWriter writer = null;
		Iterator<?> iter = ImageIO.getImageWritersByFormatName("JPEG");
		if (!iter.hasNext())
			throw new IOException("No Writers Available");
		writer = (ImageWriter) iter.next();
		ImageOutputStream ios = ImageIO.createImageOutputStream(fos);
		writer.setOutput(ios);
		JPEGImageWriteParam iwp = new JPEGImageWriteParam(null);
		iwp.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
		iwp.setCompressionQuality(0.95f);
		writer.write(null, new IIOImage(image, null, null), iwp);
		ios.flush();
		writer.dispose();
		ios.close();
	}

	public static String createNewFileName(String filename) {
		int indexOfSeparator = filename.lastIndexOf(java.io.File.separatorChar);
		String pathname = filename.substring(0, indexOfSeparator);
		String name = filename.substring(indexOfSeparator + 1);
		return pathname + java.io.File.separatorChar + "thumbnail_" + name;
	}
}
