package net.ihe.gazelle.proxy.gui;

import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Set;

import javax.ejb.Remove;
import javax.faces.context.FacesContext;
import javax.faces.model.SelectItem;
import javax.persistence.EntityManager;

import net.ihe.gazelle.common.filter.HibernateDataModel;
import net.ihe.gazelle.proxy.dao.MessageFilterStandard;
import net.ihe.gazelle.proxy.dao.ProxyDAO;
import net.ihe.gazelle.proxy.dicom.Dictionnaries;
import net.ihe.gazelle.proxy.model.message.AbstractMessage;
import net.ihe.gazelle.proxy.model.message.DicomMessage;
import net.ihe.gazelle.proxy.model.message.HL7Message;
import net.ihe.gazelle.proxy.model.message.HTTPMessage;
import net.ihe.gazelle.proxy.model.message.RawMessage;
import net.ihe.gazelle.proxy.model.message.SyslogMessage;
import net.ihe.gazelle.proxy.netty.ChannelType;
import net.ihe.gazelle.proxy.netty.channel.ProxySide;

import org.apache.xerces.impl.xpath.XPath.Step;
import org.jboss.seam.ScopeType;
import org.jboss.seam.annotations.Create;
import org.jboss.seam.annotations.Destroy;
import org.jboss.seam.annotations.In;
import org.jboss.seam.annotations.Name;
import org.jboss.seam.annotations.Scope;
import org.jboss.seam.faces.Redirect;

@Name("messagesBean")
@Scope(ScopeType.PAGE)
public class MessagesBean {

	private transient HibernateMessageDataModel<AbstractMessage> messageDataModel;
	private MessageFilterStandard messageFilterStandard;

	@In
	private EntityManager entityManager;

	private Step step = null;

	public void clearFilter() {
		messageFilterStandard.reset();
		rebuildDataModel();
	}

	@Create
	public void createModel() {
		messageFilterStandard = new MessageFilterStandard();
		rebuildDataModel();
	}

	@Remove
	@Destroy
	public void destroy() {
	}

	public Integer getConnectionId() {
		return messageFilterStandard.getConnectionId();
	}

	public Date getDateFrom() {
		return messageFilterStandard.getDateFrom();
	}

	public Date getDateTo() {
		return messageFilterStandard.getDateTo();
	}

	public String getInitiatorIP() {
		return messageFilterStandard.getInitiatorIP();
	}

	public Integer getInitiatorPort() {
		return messageFilterStandard.getInitiatorPort();
	}

	public HibernateDataModel<?> getMessageDataModel() {
		return messageDataModel;
	}

	public ChannelType getMessageType() {
		return messageFilterStandard.getMessageType();
	}

	public List<ChannelType> getMessageTypes() {
		return Arrays.asList(ChannelType.values());
	}

	public Integer getProxyPort() {
		return messageFilterStandard.getProxyPort();
	}

	public ProxySide getProxySide() {
		return messageFilterStandard.getProxySide();
	}

	public List<ProxySide> getProxySides() {
		return Arrays.asList(ProxySide.values());
	}

	public String getResponderIP() {
		return messageFilterStandard.getResponderIP();
	}

	public Integer getResponderPort() {
		return messageFilterStandard.getResponderPort();
	}

	public boolean isDICOM() {
		return messageFilterStandard.getMessageType() == ChannelType.DICOM;
	}

	public String getDicomAffectedSopClassUID() {
		return messageFilterStandard.getDicomAffectedSopClassUID();
	}

	public void setDicomAffectedSopClassUID(String dicomAffectedSopClassUID) {
		messageFilterStandard.setDicomAffectedSopClassUID(dicomAffectedSopClassUID);
	}

	public SelectItem[] getDicomAffectedSopClassUIDs() {
		Map map = Dictionnaries.INSTANCE.getSopClassByUID();
		return getSelectItems(map);
	}

	private SelectItem[] getSelectItems(Map<Object, String> map) {
		SelectItem[] result = new SelectItem[map.size() + 1];
		int i = 0;
		result[i] = new SelectItem(null, "--");
		i++;
		Set<Entry<Object, String>> entrySet = map.entrySet();
		for (Entry<Object, String> entry : entrySet) {
			result[i] = new SelectItem(entry.getKey(), entry.getValue());
			i++;
		}
		return result;
	}

	public String getDicomRequestedSopClassUID() {
		return messageFilterStandard.getDicomRequestedSopClassUID();
	}

	public void setDicomRequestedSopClassUID(String dicomRequestedSopClassUID) {
		messageFilterStandard.setDicomRequestedSopClassUID(dicomRequestedSopClassUID);
	}

	public SelectItem[] getDicomRequestedSopClassUIDs() {
		return getDicomAffectedSopClassUIDs();
	}

	public String getDicomCommandField() {
		return messageFilterStandard.getDicomCommandField();
	}

	public void setDicomCommandField(String dicomCommandField) {
		messageFilterStandard.setDicomCommandField(dicomCommandField);
	}

	public SelectItem[] getDicomCommandFields() {
		Map map = Dictionnaries.INSTANCE.getCommandFields();
		return getSelectItems(map);
	}

	private void rebuildDataModel() {
		messageDataModel = new HibernateMessageDataModel<AbstractMessage>(messageFilterStandard);
	}

	public void redirectToMessageFromURL() {
		FacesContext fc = FacesContext.getCurrentInstance();
		String id = (String) fc.getExternalContext().getRequestParameterMap().get("id");
		AbstractMessage message = ProxyDAO.getMessageByID(Integer.parseInt(id));

		Redirect redirect = Redirect.instance();
		redirect.setParameter("id", id);

		String viewId = "";
		if (message instanceof HL7Message) {
			viewId = "/messages/hl7.xhtml";
		}
		if (message instanceof RawMessage) {
			viewId = "/messages/raw.xhtml";
		}
		if (message instanceof DicomMessage) {
			viewId = "/messages/dicom.xhtml";
		}
		if (message instanceof SyslogMessage) {
			viewId = "/messages/syslog.xhtml";
		}
		if (message instanceof HTTPMessage) {
			viewId = "/messages/http.xhtml";
		}

		redirect.setViewId(viewId);
		redirect.execute();
	}

	public String redirectToMessage(AbstractMessage message) {
		if (message.getId() == null) {
			return null;
		}
		if (message instanceof HL7Message) {
			return "/messages/hl7.seam?id=" + message.getId();
		}
		if (message instanceof RawMessage) {
			return "/messages/raw.seam?id=" + message.getId();
		}
		if (message instanceof DicomMessage) {
			return "/messages/dicom.seam?id=" + message.getId();
		}
		if (message instanceof SyslogMessage) {
			return "/messages/syslog.seam?id=" + message.getId();
		}
		if (message instanceof HTTPMessage) {
			return "/messages/http.seam?id=" + message.getId();
		}
		return null;
	}

	public void setConnectionId(Integer connectionId) {
		messageFilterStandard.setConnectionId(connectionId);
	}

	public void setDateFrom(Date dateFrom) {
		messageFilterStandard.setDateFrom(dateFrom);
	}

	public void setDateTo(Date dateTo) {
		messageFilterStandard.setDateTo(dateTo);
	}

	public void setInitiatorIP(String initiatorIP) {
		messageFilterStandard.setInitiatorIP(initiatorIP);
	}

	public void setInitiatorPort(Integer initiatorPort) {
		messageFilterStandard.setInitiatorPort(initiatorPort);
	}

	public void setMessageType(ChannelType messageType) {
		messageFilterStandard.setMessageType(messageType);
		rebuildDataModel();
	}

	public void setProxyPort(Integer proxyPort) {
		messageFilterStandard.setProxyPort(proxyPort);
	}

	public void setProxySide(ProxySide proxySide) {
		messageFilterStandard.setProxySide(proxySide);
	}

	public void setResponderIP(String responderIP) {
		messageFilterStandard.setResponderIP(responderIP);
	}

	public void setResponderPort(Integer responderPort) {
		messageFilterStandard.setResponderPort(responderPort);
	}

}
